import numpy as np

def f(x):
    return x**2 * np.exp(-x) - x

def grad_f(x):
    return (2 - x) * x * np.exp(-x)

def armijo_grad_desc(x0, alpha=0.5, beta=0.5, epsilon=1e-6, max_iter=1000):
    i = 0
    x = x0
    while i < max_iter:
        grad = grad_f(x)
        t = 1
        while f(x - t*grad) > f(x) - alpha*t*np.linalg.norm(grad)**2:
            t *= beta
        x = x - t*grad
        if np.linalg.norm(grad) < epsilon:
            break
        i += 1
    return x, f(x), i